@@ -1,21 +1,24 @@
+use strict; use warnings;
 package IO::All;
-use 5.006001;
-use strict;
-use warnings;
+our $VERSION = '0.86';
+
 require Carp;
 # So one can use Carp::carp "$message" - without the parenthesis.
 sub Carp::carp;
+
 use IO::All::Base -base;
-our $VERSION = '0.41';
+
 use File::Spec();
 use Symbol();
 use Fcntl;
+use Cwd ();
+
 our @EXPORT = qw(io);
 
 #===============================================================================
 # Object creation and setup methods
 #===============================================================================
-my $autoload = { 
+my $autoload = {
     qw(
         touch file
 
@@ -58,27 +61,29 @@ sub AUTOLOAD {
     $method =~ s/.*:://;
     my $pkg = ref($self) || $self;
     $self->throw(qq{Can't locate object method "$method" via package "$pkg"})
-      if $pkg ne $self->package;
-    my $class = $self->autoload_class($method);
+      if $pkg ne $self->_package;
+    my $class = $self->_autoload_class($method);
     my $foo = "$self";
     bless $self, $class;
     $self->$method(@_);
 }
 
-sub autoload_class {
+sub _autoload_class {
     my $self = shift;
     my $method = shift;
     my $class_id = $self->autoload->{$method} || $method;
-    return "IO::All::\u$class_id" if $INC{"IO/All/\u$class_id\E.pm"};
+    my $ucfirst_class_name = 'IO::All::' . ucfirst($class_id);
+    my $ucfirst_class_fn = "IO/All/" . ucfirst($class_id) . ".pm";
+    return $ucfirst_class_name if $INC{$ucfirst_class_fn};
     return "IO::All::\U$class_id" if $INC{"IO/All/\U$class_id\E.pm"};
     require IO::All::Temp;
-    if (eval "require IO::All::\u$class_id; 1") {
-        my $class = "IO::All::\u$class_id";
+    if (eval "require $ucfirst_class_name; 1") {
+        my $class = $ucfirst_class_name;
         my $return = $class->can('new')
         ? $class
         : do { # (OS X hack)
-            my $value = $INC{"IO/All/\u$class_id\E.pm"};
-            delete $INC{"IO/All/\u$class_id\E.pm"};
+            my $value = $INC{$ucfirst_class_fn};
+            delete $INC{$ucfirst_class_fn};
             $INC{"IO/All/\U$class_id\E.pm"} = $value;
             "IO::All::\U$class_id";
         };
@@ -94,7 +99,7 @@ sub new {
     my $self = shift;
     my $package = ref($self) || $self;
     my $new = bless Symbol::gensym(), $package;
-    $new->package($package);
+    $new->_package($package);
     $new->_copy_from($self) if ref($self);
     my $name = shift;
     return $name if UNIVERSAL::isa($name, 'IO::All');
@@ -108,7 +113,7 @@ sub new {
     return $new->dir($name) if -d $name;
     return $new->$1($name) if $name =~ /^([a-z]{3,8}):/;
     return $new->socket($name) if $name =~ /^[\w\-\.]*:\d{1,5}$/;
-    return $new->pipe($name) 
+    return $new->pipe($name)
       if $name =~ s/^\s*\|\s*// or $name =~ s/\s*\|\s*$//;
     return $new->string if $name eq '$';
     return $new->stdio if $name eq '-';
@@ -134,90 +139,41 @@ sub handle {
     return $self->_init;
 }
 
-#===============================================================================
-# Tie Interface
-#===============================================================================
-sub tie {
-    my $self = shift; 
-    tie *$self, $self; 
-    return $self;
-}
-
-sub TIEHANDLE {
-    return $_[0] if ref $_[0];
-    my $class = shift;
-    my $self = bless Symbol::gensym(), $class;
-    $self->init(@_);
-}
-
-sub READLINE {
-    goto &getlines if wantarray;
-    goto &getline;
-}
-
-sub DESTROY {
-    my $self = shift;
-    no warnings;
-    unless ( $^V and $^V lt v5.8.0 ) {
-        untie *$self if tied *$self;
-    }
-    $self->close if $self->is_open;
-}
-
-sub BINMODE {
-    my $self = shift; 
-    binmode *$self->io_handle;
-}
-
-{
-    no warnings;
-    *GETC   = \&getc;
-    *PRINT  = \&print;
-    *PRINTF = \&printf;
-    *READ   = \&read;
-    *WRITE  = \&write;
-    *SEEK   = \&seek;
-    *TELL   = \&getpos;
-    *EOF    = \&eof;
-    *CLOSE  = \&close;
-    *FILENO = \&fileno;
-}
-
 #===============================================================================
 # Overloading support
 #===============================================================================
-my $old_warn_handler = $SIG{__WARN__}; 
-$SIG{__WARN__} = sub { 
+my $old_warn_handler = $SIG{__WARN__};
+$SIG{__WARN__} = sub {
     if ($_[0] !~ /^Useless use of .+ \(.+\) in void context/) {
         goto &$old_warn_handler if $old_warn_handler;
         warn(@_);
     }
 };
-    
-use overload '""' => 'overload_stringify';
-use overload '|' => 'overload_bitwise_or';
-use overload '<<' => 'overload_left_bitshift';
-use overload '>>' => 'overload_right_bitshift';
-use overload '<' => 'overload_less_than';
-use overload '>' => 'overload_greater_than';
-use overload '${}' => 'overload_string_deref';
-use overload '@{}' => 'overload_array_deref';
-use overload '%{}' => 'overload_hash_deref';
-use overload '&{}' => 'overload_code_deref';
 
-sub overload_bitwise_or {my $self = shift; $self->overload_handler(@_, '|') }
-sub overload_left_bitshift {my $self = shift; $self->overload_handler(@_, '<<') }
-sub overload_right_bitshift {my $self = shift; $self->overload_handler(@_, '>>') }
-sub overload_less_than {my $self = shift; $self->overload_handler(@_, '<') }
-sub overload_greater_than {my $self = shift; $self->overload_handler(@_, '>') }
-sub overload_string_deref {my $self = shift; $self->overload_handler(@_, '${}') }
-sub overload_array_deref {my $self = shift; $self->overload_handler(@_, '@{}') }
-sub overload_hash_deref {my $self = shift; $self->overload_handler(@_, '%{}') }
-sub overload_code_deref {my $self = shift; $self->overload_handler(@_, '&{}') }
+use overload '""' => '_overload_stringify';
+use overload '|' => '_overload_bitwise_or';
+use overload '<<' => '_overload_left_bitshift';
+use overload '>>' => '_overload_right_bitshift';
+use overload '<' => '_overload_less_than';
+use overload '>' => '_overload_greater_than';
+use overload '${}' => '_overload_string_deref';
+use overload '@{}' => '_overload_array_deref';
+use overload '%{}' => '_overload_hash_deref';
+use overload '&{}' => '_overload_code_deref';
 
-sub overload_handler {
+sub _overload_bitwise_or {my $self = shift; $self->_overload_handler(@_, '|') }
+sub _overload_left_bitshift {my $self = shift; $self->_overload_handler(@_, '<<') }
+sub _overload_right_bitshift {my $self = shift; $self->_overload_handler(@_, '>>') }
+sub _overload_less_than {my $self = shift; $self->_overload_handler(@_, '<') }
+sub _overload_greater_than {my $self = shift; $self->_overload_handler(@_, '>') }
+sub _overload_string_deref {my $self = shift; $self->_overload_handler(@_, '${}') }
+sub _overload_array_deref {my $self = shift; $self->_overload_handler(@_, '@{}') }
+sub _overload_hash_deref {my $self = shift; $self->_overload_handler(@_, '%{}') }
+sub _overload_code_deref {my $self = shift; $self->_overload_handler(@_, '&{}') }
+
+sub _overload_handler {
     my ($self) = @_;
-    my $method = $self->get_overload_method(@_);
+    my $method = $self->_get_overload_method(@_);
     $self->$method(@_);
 }
 
@@ -226,40 +182,40 @@ my $op_swap = {
     '<' => '>', '<<' => '>>',
 };
 
-sub overload_table {
+sub _overload_table {
     my $self = shift;
     (
-        '* > *' => 'overload_any_to_any',
-        '* < *' => 'overload_any_from_any',
-        '* >> *' => 'overload_any_addto_any',
-        '* << *' => 'overload_any_addfrom_any',
-     
-        '* < scalar' => 'overload_scalar_to_any',
-        '* > scalar' => 'overload_any_to_scalar',
-        '* << scalar' => 'overload_scalar_addto_any',
-        '* >> scalar' => 'overload_any_addto_scalar',
+        '* > *' => '_overload_any_to_any',
+        '* < *' => '_overload_any_from_any',
+        '* >> *' => '_overload_any_addto_any',
+        '* << *' => '_overload_any_addfrom_any',
+
+        '* < scalar' => '_overload_scalar_to_any',
+        '* > scalar' => '_overload_any_to_scalar',
+        '* << scalar' => '_overload_scalar_addto_any',
+        '* >> scalar' => '_overload_any_addto_scalar',
     )
 };
 
-sub get_overload_method {
+sub _get_overload_method {
     my ($self, $arg1, $arg2, $swap, $operator) = @_;
     if ($swap) {
         $operator = $op_swap->{$operator} || $operator;
     }
-    my $arg1_type = $self->get_argument_type($arg1);
-    my $table1 = { $arg1->overload_table };
+    my $arg1_type = $self->_get_argument_type($arg1);
+    my $table1 = { $arg1->_overload_table };
 
     if ($operator =~ /\{\}$/) {
         my $key = "$operator $arg1_type";
-        return $table1->{$key} || $self->overload_undefined($key);
+        return $table1->{$key} || $self->_overload_undefined($key);
     }
-    
-    my $arg2_type = $self->get_argument_type($arg2);
-    my @table2 = UNIVERSAL::isa($arg2, "IO::All") 
-    ? ($arg2->overload_table) 
+
+    my $arg2_type = $self->_get_argument_type($arg2);
+    my @table2 = UNIVERSAL::isa($arg2, "IO::All")
+    ? ($arg2->_overload_table)
     : ();
     my $table = { %$table1, @table2 };
-     
+
     my @keys = (
         "$arg1_type $operator $arg2_type",
         "* $operator $arg2_type",
@@ -268,14 +224,14 @@ sub get_overload_method {
       unless $arg2_type =~ /^(scalar|array|hash|code|ref)$/;
 
     for (@keys) {
-        return $table->{$_} 
+        return $table->{$_}
           if defined $table->{$_};
     }
 
-    return $self->overload_undefined($keys[0]);
+    return $self->_overload_undefined($keys[0]);
 }
 
-sub get_argument_type {
+sub _get_argument_type {
     my $self = shift;
     my $argument = shift;
     my $ref = ref($argument);
@@ -289,63 +245,63 @@ sub get_argument_type {
     return $argument->type || 'unknown';
 }
 
-sub overload_stringify {
+sub _overload_stringify {
     my $self = shift;
     my $name = $self->pathname;
     return defined($name) ? $name : overload::StrVal($self);
 }
 
-sub overload_undefined {
+sub _overload_undefined {
     my $self = shift;
     require Carp;
     my $key = shift;
     Carp::carp "Undefined behavior for overloaded IO::All operation: '$key'"
       if $^W;
-    return 'overload_noop';
+    return '_overload_noop';
 }
 
-sub overload_noop {
+sub _overload_noop {
     my $self = shift;
     return;
 }
 
-sub overload_any_addfrom_any {
+sub _overload_any_addfrom_any {
     $_[1]->append($_[2]->all);
     $_[1];
 }
 
-sub overload_any_addto_any {
+sub _overload_any_addto_any {
     $_[2]->append($_[1]->all);
     $_[2];
 }
 
-sub overload_any_from_any {
+sub _overload_any_from_any {
     $_[1]->close if $_[1]->is_file and $_[1]->is_open;
     $_[1]->print($_[2]->all);
     $_[1];
 }
 
-sub overload_any_to_any {
+sub _overload_any_to_any {
     $_[2]->close if $_[2]->is_file and $_[2]->is_open;
     $_[2]->print($_[1]->all);
     $_[2];
 }
 
-sub overload_any_to_scalar {
+sub _overload_any_to_scalar {
     $_[2] = $_[1]->all;
 }
 
-sub overload_any_addto_scalar {
+sub _overload_any_addto_scalar {
     $_[2] .= $_[1]->all;
     $_[2];
 }
 
-sub overload_scalar_addto_any {
+sub _overload_scalar_addto_any {
     $_[1]->append($_[2]);
     $_[1];
 }
 
-sub overload_scalar_to_any {
+sub _overload_scalar_to_any {
     local $\;
     $_[1]->close if $_[1]->is_file and $_[1]->is_open;
     $_[1]->print($_[2]);
@@ -355,18 +311,16 @@ sub overload_scalar_to_any {
 #===============================================================================
 # Private Accessors
 #===============================================================================
-field 'package';
-field _binary => undef;
-field _binmode => undef;
+field '_package';
 field _strict => undef;
-field _encoding => undef;
-field _utf8 => undef;
+field _layers => [];
 field _handle => undef;
+field _constructor => undef;
+field _partial_spec_class => undef;
 
 #===============================================================================
 # Public Accessors
 #===============================================================================
-field constructor => undef;
 chain block_size => 1024;
 chain errors => undef;
 field io_handle => undef;
@@ -376,6 +330,19 @@ chain name => undef;
 chain perms => undef;
 chain separator => $/;
 field type => '';
+
+sub _spec_class {
+   my $self = shift;
+
+   my $ret = 'File::Spec';
+   if (my $partial = $self->_partial_spec_class(@_)) {
+      $ret .= '::' . $partial;
+      eval "require $ret";
+   }
+
+   return $ret
+}
+
 sub pathname {my $self = shift; $self->name(@_) }
 
 #===============================================================================
@@ -411,69 +378,122 @@ proxy_open syswrite => O_CREAT | O_WRONLY;
 proxy_open seek => $^O eq 'MSWin32' ? '<' : '+<';
 proxy_open 'getc';
 
+#===============================================================================
+# Tie Interface
+#===============================================================================
+sub tie {
+    my $self = shift;
+    tie *$self, $self;
+    return $self;
+}
+
+sub TIEHANDLE {
+    return $_[0] if ref $_[0];
+    my $class = shift;
+    my $self = bless Symbol::gensym(), $class;
+    $self->init(@_);
+}
+
+sub READLINE {
+    goto &getlines if wantarray;
+    goto &getline;
+}
+
+sub DESTROY {
+    my $self = shift;
+    no warnings;
+    unless ( $] < 5.008 ) {
+        untie *$self if tied *$self;
+    }
+    $self->close if $self->is_open;
+}
+
+sub BINMODE {
+    my $self = shift;
+    CORE::binmode *$self->io_handle;
+}
+
+{
+    no warnings;
+    *GETC   = \&getc;
+    *PRINT  = \&print;
+    *PRINTF = \&printf;
+    *READ   = \&read;
+    *WRITE  = \&write;
+    *SEEK   = \&seek;
+    *TELL   = \&getpos;
+    *EOF    = \&eof;
+    *CLOSE  = \&close;
+    *FILENO = \&fileno;
+}
+
 #===============================================================================
 # File::Spec Interface
 #===============================================================================
-sub canonpath {my $self = shift; File::Spec->canonpath($self->pathname) } 
+sub canonpath {my $self = shift;
+   eval { Cwd::abs_path($self->pathname); 0 } ||
+      File::Spec->canonpath($self->pathname)
+}
+
 sub catdir {
     my $self = shift;
     my @args = grep defined, $self->name, @_;
-    $self->constructor->()->dir(File::Spec->catdir(@args));
-} 
+    $self->_constructor->()->dir(File::Spec->catdir(@args));
+}
 sub catfile {
     my $self = shift;
     my @args = grep defined, $self->name, @_;
-    $self->constructor->()->file(File::Spec->catfile(@args));
-} 
-sub join {my $self = shift; $self->catfile(@_) } 
+    $self->_constructor->()->file(File::Spec->catfile(@args));
+}
+sub join {my $self = shift; $self->catfile(@_) }
 sub curdir {
     my $self = shift;
-    $self->constructor->()->dir(File::Spec->curdir);
-} 
+    $self->_constructor->()->dir(File::Spec->curdir);
+}
 sub devnull {
     my $self = shift;
-    $self->constructor->()->file(File::Spec->devnull);
-} 
+    $self->_constructor->()->file(File::Spec->devnull);
+}
 sub rootdir {
     my $self = shift;
-    $self->constructor->()->dir(File::Spec->rootdir);
-} 
+    $self->_constructor->()->dir(File::Spec->rootdir);
+}
 sub tmpdir {
     my $self = shift;
-    $self->constructor->()->dir(File::Spec->tmpdir);
-} 
+    $self->_constructor->()->dir(File::Spec->tmpdir);
+}
 sub updir {
     my $self = shift;
-    $self->constructor->()->dir(File::Spec->updir);
-} 
+    $self->_constructor->()->dir(File::Spec->updir);
+}
 sub case_tolerant {
     my $self = shift;
     File::Spec->case_tolerant;
-} 
+}
 sub is_absolute {
     my $self = shift;
     File::Spec->file_name_is_absolute($self->pathname);
 }
 sub path {
     my $self = shift;
-    map { $self->constructor->()->dir($_) } File::Spec->path;
-} 
+    map { $self->_constructor->()->dir($_) } File::Spec->path;
+}
 sub splitpath {
     my $self = shift;
     File::Spec->splitpath($self->pathname);
-} 
+}
 sub splitdir {
     my $self = shift;
     File::Spec->splitdir($self->pathname);
-} 
+}
 sub catpath {
     my $self = shift;
-    $self->constructor->(File::Spec->catpath(@_));
-} 
+    $self->_constructor->(File::Spec->catpath(@_));
+}
 sub abs2rel {
     my $self = shift;
     File::Spec->abs2rel($self->pathname, @_);
-} 
+}
 sub rel2abs {
     my $self = shift;
     File::Spec->rel2abs($self->pathname, @_);
@@ -482,48 +502,58 @@ sub rel2abs {
 #===============================================================================
 # Public IO Action Methods
 #===============================================================================
+sub absolute {
+    my $self = shift;
+    $self->pathname(File::Spec->rel2abs($self->pathname))
+      unless $self->is_absolute;
+    $self->is_absolute(1);
+    return $self;
+}
+
 sub all {
     my $self = shift;
-    $self->assert_open('<');
+    $self->_assert_open('<');
     local $/;
     my $all = $self->io_handle->getline;
-    $self->error_check;
+    $self->_error_check;
     $self->_autoclose && $self->close;
     return $all;
 }
 
 sub append {
     my $self = shift;
-    $self->assert_open('>>');
+    $self->_assert_open('>>');
     $self->print(@_);
 }
 
 sub appendln {
     my $self = shift;
-    $self->assert_open('>>');
+    $self->_assert_open('>>');
     $self->println(@_);
 }
 
 sub binary {
     my $self = shift;
-    binmode($self->io_handle)
-      if $self->is_open;
-    $self->_binary(1);
+    CORE::binmode($self->io_handle) if $self->is_open;
+    push @{$self->_layers}, ":raw";
     return $self;
 }
 
 sub binmode {
     my $self = shift;
     my $layer = shift;
-    if ($self->is_open) {
-        $layer
-        ? CORE::binmode($self->io_handle, $layer)
-        : CORE::binmode($self->io_handle);
-    }
-    $self->_binmode($layer);
+    $self->_sane_binmode($layer) if $self->is_open;
+    push @{$self->_layers}, $layer;
     return $self;
 }
 
+sub _sane_binmode {
+    my ($self, $layer) = @_;
+    $layer
+    ? CORE::binmode($self->io_handle, $layer)
+    : CORE::binmode($self->io_handle);
+}
+
 sub buffer {
     my $self = shift;
     if (not @_) {
@@ -569,14 +599,14 @@ sub getline {
     my $self = shift;
     return $self->getline_backwards
       if $self->_backwards;
-    $self->assert_open('<');
+    $self->_assert_open('<');
     my $line;
     {
         local $/ = @_ ? shift(@_) : $self->separator;
         $line = $self->io_handle->getline;
         chomp($line) if $self->_chomp and defined $line;
     }
-    $self->error_check;
+    $self->_error_check;
     return $line if defined $line;
     $self->close if $self->_autoclose;
     return undef;
@@ -586,7 +616,7 @@ sub getlines {
     my $self = shift;
     return $self->getlines_backwards
       if $self->_backwards;
-    $self->assert_open('<');
+    $self->_assert_open('<');
     my @lines;
     {
         local $/ = @_ ? shift(@_) : $self->separator;
@@ -595,10 +625,10 @@ sub getlines {
             chomp for @lines;
         }
     }
-    $self->error_check;
-    return (@lines) or
-           $self->_autoclose && $self->close && () or
-           ();
+    $self->_error_check;
+    return @lines if @lines;
+    $self->close if $self->_autoclose;
+    return ();
 }
 
 sub is_dir {my $self = shift; UNIVERSAL::isa($self, 'IO::All::Dir') }
@@ -639,7 +669,7 @@ sub open {
         # XXX Not tested
         $self->io_handle->fdopen($self->_handle, @args);
     }
-    $self->set_binmode;
+    $self->_set_binmode;
 }
 
 sub println {
@@ -649,15 +679,15 @@ sub println {
 
 sub read {
     my $self = shift;
-    $self->assert_open('<');
+    $self->_assert_open('<');
     my $length = (@_ or $self->type eq 'dir')
     ? $self->io_handle->read(@_)
     : $self->io_handle->read(
-        ${$self->buffer}, 
-        $self->block_size, 
+        ${$self->buffer},
+        $self->block_size,
         $self->length,
     );
-    $self->error_check;
+    $self->_error_check;
     return $length || $self->_autoclose && $self->close && 0;
 }
 
@@ -680,7 +710,7 @@ sub slurp {
     if ($self->_chomp) {
         local $/ = $separator;
         map {chomp; $_} split /(?<=\Q$separator\E)/, $slurp;
-    }   
+    }
     else {
         split /(?<=\Q$separator\E)/, $slurp;
     }
@@ -691,33 +721,38 @@ sub utf8 {
     if ($] < 5.008) {
         die "IO::All -utf8 not supported on Perl older than 5.8";
     }
-    CORE::binmode($self->io_handle, ':utf8')
-      if $self->is_open;
-    $self->_utf8(1);
-    $self->encoding('utf8');
+    $self->encoding('UTF-8');
     return $self;
 }
 
+sub _has_utf8 {
+    grep { $_ eq ':encoding(UTF-8)' } @{shift->_layers}
+}
+
 sub encoding {
     my $self = shift;
-    my $encoding = shift
-      or die "No encoding value passed to IO::All::encoding";
+    my $encoding = shift;
     if ($] < 5.008) {
         die "IO::All -encoding not supported on Perl older than 5.8";
     }
-    CORE::binmode($self->io_handle, ":$encoding")
-      if $self->is_open;
-    $self->_encoding($encoding);
+    die "No valid encoding string sent" if !$encoding;
+    $self->_set_encoding($encoding) if $self->is_open and $encoding;
+    push @{$self->_layers}, ":encoding($encoding)";
     return $self;
 }
 
+sub _set_encoding {
+    my ($self, $encoding) = @_;
+    return CORE::binmode($self->io_handle, ":encoding($encoding)");
+}
+
 sub write {
     my $self = shift;
-    $self->assert_open('>');
+    $self->_assert_open('>');
     my $length = @_
     ? $self->io_handle->write(@_)
     : $self->io_handle->write(${$self->buffer}, $self->length);
-    $self->error_check;
+    $self->_error_check;
     $self->clear unless @_;
     return $length;
 }
@@ -739,72 +774,57 @@ sub throw {
 #===============================================================================
 # Private instance methods
 #===============================================================================
-sub assert_dirpath {
+sub _assert_dirpath {
     my $self = shift;
     my $dir_name = shift;
-    return $dir_name if -d $dir_name or
-      CORE::mkdir($self->pathname, $self->perms || 0755) or
+    return $dir_name if ((! CORE::length($dir_name)) or
+      -d $dir_name or
+      CORE::mkdir($dir_name, $self->perms || 0755) or
       do {
           require File::Path;
-          File::Path::mkpath($dir_name);
+          File::Path::mkpath($dir_name, 0, $self->perms || 0755 );
       } or
-      $self->throw("Can't make $dir_name"); 
+      $self->throw("Can't make $dir_name"));
 }
 
-sub assert_open {
+sub _assert_open {
     my $self = shift;
     return if $self->is_open;
     $self->file unless $self->type;
     return $self->open(@_);
 }
 
-sub error_check {
+sub _error_check {
     my $self = shift;
     return unless $self->io_handle->can('error');
     return unless $self->io_handle->error;
     $self->throw($!);
 }
 
-sub copy {
+sub _set_binmode {
     my $self = shift;
-    my $copy;
-    for (keys %{*$self}) {
-        $copy->{$_} = *$self->{$_};
-    }
-    $copy->{io_handle} = 'defined'
-      if defined $copy->{io_handle};
-    return $copy;
-}
-
-sub set_binmode {
-    my $self = shift;
-    if (my $encoding = $self->_encoding) {
-        CORE::binmode($self->io_handle, ":encoding($encoding)");
-    }
-    elsif ($self->_binary) {
-        CORE::binmode($self->io_handle);
-    }
-    elsif ($self->_binmode) {
-        CORE::binmode($self->io_handle, $self->_binmode);
-    }
+    $self->_sane_binmode($_) for @{$self->_layers};
     return $self;
 }
 
 #===============================================================================
 # Stat Methods
 #===============================================================================
-sub device    {my $self = shift; my $x = (stat($self->io_handle || $self->pathname))[0] }
-sub inode     {my $self = shift; my $x = (stat($self->io_handle || $self->pathname))[1] }
-sub modes     {my $self = shift; my $x = (stat($self->io_handle || $self->pathname))[2] }
-sub nlink     {my $self = shift; my $x = (stat($self->io_handle || $self->pathname))[3] }
-sub uid       {my $self = shift; my $x = (stat($self->io_handle || $self->pathname))[4] }
-sub gid       {my $self = shift; my $x = (stat($self->io_handle || $self->pathname))[5] }
-sub device_id {my $self = shift; my $x = (stat($self->io_handle || $self->pathname))[6] }
-sub size      {my $self = shift; my $x = (stat($self->io_handle || $self->pathname))[7] }
-sub atime     {my $self = shift; my $x = (stat($self->io_handle || $self->pathname))[8] }
-sub mtime     {my $self = shift; my $x = (stat($self->io_handle || $self->pathname))[9] }
-sub ctime     {my $self = shift; my $x = (stat($self->io_handle || $self->pathname))[10] }
-sub blksize   {my $self = shift; my $x = (stat($self->io_handle || $self->pathname))[11] }
-sub blocks    {my $self = shift; my $x = (stat($self->io_handle || $self->pathname))[12] }
+BEGIN {
+    no strict 'refs';
+    my @stat_fields = qw(
+        device inode modes nlink uid gid device_id size atime mtime
+        ctime blksize blocks
+    );
+    foreach my $stat_field_idx (0 .. $#stat_fields)
+    {
+        my $idx = $stat_field_idx;
+        my $name = $stat_fields[$idx];
+
+        *$name = sub {
+            my $self = shift;
+            return (stat($self->io_handle || $self->pathname))[$idx];
+        };
+    }
+}
 
-1;
